@@ -34,9 +34,6 @@ LOG = logging.getLogger(__name__)
 
 
 A10_GLOBAL_OPTS = [
-    cfg.BoolOpt('enable_hierarchical_multitenancy', default=False,
-                help=_('Enable Hierarchical Multitenancy '
-                       'for racked - hardware thunder devices.')),
     cfg.BoolOpt('use_parent_partition', default=False,
                 help=_('Use parent project partition on Thunder device '
                        'in hierarchical project architecture.')),
@@ -48,6 +45,9 @@ A10_GLOBAL_OPTS = [
                default=a10constants.FLAT,
                choices=a10constants.SUPPORTED_NETWORK_TYPE,
                help=_('Neutron ML2 Tenent Network Type')),
+    cfg.BoolOpt('use_shared_for_template_lookup',
+                default=False,
+                help=_('Use shared for template')),
 ]
 
 A10_VTHUNDER_OPTS = [
@@ -60,7 +60,6 @@ A10_VTHUNDER_OPTS = [
     cfg.IntOpt('default_axapi_version',
                default=30,
                help=_('VThunder axapi version')),
-
 ]
 
 A10_SLB_OPTS = [
@@ -78,7 +77,7 @@ A10_LISTENER_OPTS = [
                 default=False,
                 help=_('Disable destination NAT')),
     cfg.BoolOpt('ha_conn_mirror',
-                default=False,
+                default=None,
                 help=_('Enable for HA Conn sync')),
     cfg.StrOpt('template_virtual_port',
                default=None,
@@ -130,7 +129,7 @@ A10_SERVER_OPTS = [
                default=64000000,
                help=_('Connection Limit')),
     cfg.IntOpt('conn_resume', min=1, max=1000000,
-               default=1,
+               default=None,
                help=_('Connection Resume')),
     cfg.StrOpt('template_server',
                default=None,
