@@ -138,7 +138,7 @@ class Thunder(BaseDataModel):
                  loadbalancer_id=None, project_id=None, compute_id=None,
                  topology="STANDALONE", role="MASTER", last_udp_update=None, status="ACTIVE",
                  created_at=datetime.utcnow(), updated_at=datetime.utcnow(),
-                 partition_name="shared", hierarchical_multitenancy=None,
+                 partition_name="shared", hierarchical_multitenancy="disable",
                  vrid_floating_ip=None, device_network_map=None):
         self.id = id
         self.vthunder_id = vthunder_id
@@ -189,12 +189,13 @@ class Certificate(BaseDataModel):
 class VRID(BaseDataModel):
 
     def __init__(self, id=None, project_id=None, vrid=None, vrid_port_id=None,
-                 vrid_floating_ip=None):
+                 vrid_floating_ip=None, subnet_id=None):
         self.id = id
         self.project_id = project_id
         self.vrid = vrid
         self.vrid_port_id = vrid_port_id
         self.vrid_floating_ip = vrid_floating_ip
+        self.subnet_id = subnet_id
 
 
 class Interface(BaseDataModel):
