@@ -28,6 +28,7 @@ from octavia.db import api as db_apis
 from octavia.db import repositories as repo
 
 from a10_octavia.common import a10constants
+from a10_octavia.common import utils
 from a10_octavia.controller.worker.flows import a10_health_monitor_flows
 from a10_octavia.controller.worker.flows import a10_l7policy_flows
 from a10_octavia.controller.worker.flows import a10_l7rule_flows
@@ -114,7 +115,8 @@ class A10ControllerWorker(base_taskflow.BaseTaskFlowEngine):
             store={constants.HEALTH_MON: health_mon,
                    constants.POOL: pool,
                    constants.LISTENERS: listeners,
-                   constants.LOADBALANCER: load_balancer})
+                   constants.LOADBALANCER: load_balancer,
+                   a10constants.WRITE_MEM_SHARED_PART: True})
         with tf_logging.DynamicLoggingListener(create_hm_tf,
                                                log=LOG):
             create_hm_tf.run()
@@ -138,7 +140,8 @@ class A10ControllerWorker(base_taskflow.BaseTaskFlowEngine):
             store={constants.HEALTH_MON: health_mon,
                    constants.POOL: pool,
                    constants.LISTENERS: listeners,
-                   constants.LOADBALANCER: load_balancer})
+                   constants.LOADBALANCER: load_balancer,
+                   a10constants.WRITE_MEM_SHARED_PART: True})
         with tf_logging.DynamicLoggingListener(delete_hm_tf,
                                                log=LOG):
             delete_hm_tf.run()
@@ -174,7 +177,8 @@ class A10ControllerWorker(base_taskflow.BaseTaskFlowEngine):
                    constants.POOL: pool,
                    constants.LISTENERS: listeners,
                    constants.LOADBALANCER: load_balancer,
-                   constants.UPDATE_DICT: health_monitor_updates})
+                   constants.UPDATE_DICT: health_monitor_updates,
+                   a10constants.WRITE_MEM_SHARED_PART: True})
         with tf_logging.DynamicLoggingListener(update_hm_tf,
                                                log=LOG):
             update_hm_tf.run()
@@ -190,21 +194,26 @@ class A10ControllerWorker(base_taskflow.BaseTaskFlowEngine):
             raise db_exceptions.NoResultFound
 
         load_balancer = listener.load_balancer
-        if listener.project_id in CONF.hardware_thunder.devices:
+        parent_project_list = utils.get_parent_project_list()
+        listener_parent_proj = utils.get_parent_project(listener.project_id)
+
+        if (listener.project_id in parent_project_list or
+                (listener_parent_proj and listener_parent_proj in parent_project_list)
+                or listener.project_id in CONF.hardware_thunder.devices):
             create_listener_tf = self._taskflow_load(self._listener_flows.
                                                      get_rack_vthunder_create_listener_flow(
                                                          listener.project_id),
                                                      store={constants.LOADBALANCER:
                                                             load_balancer,
                                                             constants.LISTENER:
-                                                                listener})
+                                                            listener})
         else:
             create_listener_tf = self._taskflow_load(self._listener_flows.
                                                      get_create_listener_flow(),
                                                      store={constants.LOADBALANCER:
                                                             load_balancer,
                                                             constants.LISTENER:
-                                                                listener})
+                                                            listener})
 
         with tf_logging.DynamicLoggingListener(create_listener_tf,
                                                log=LOG):
@@ -216,6 +225,7 @@ class A10ControllerWorker(base_taskflow.BaseTaskFlowEngine):
         listener = self._listener_repo.get(db_apis.get_session(),
                                            id=listener_id)
         load_balancer = listener.load_balancer
+
         if listener.project_id in CONF.hardware_thunder.devices:
             delete_listener_tf = self._taskflow_load(
                 self._listener_flows.get_delete_rack_listener_flow(),
@@ -274,6 +284,7 @@ class A10ControllerWorker(base_taskflow.BaseTaskFlowEngine):
             raise db_exceptions.NoResultFound
 
         store = {constants.LOADBALANCER_ID: load_balancer_id,
+                 constants.VIP: lb.vip,
                  constants.BUILD_TYPE_PRIORITY:
                  constants.LB_CREATE_NORMAL_PRIORITY}
 
@@ -282,6 +293,7 @@ class A10ControllerWorker(base_taskflow.BaseTaskFlowEngine):
         store[constants.UPDATE_DICT] = {
             constants.TOPOLOGY: topology
         }
+
         if lb.project_id in CONF.hardware_thunder.devices:
             create_lb_flow = self._lb_flows.get_create_rack_vthunder_load_balancer_flow(
                 vthunder_conf=CONF.hardware_thunder.devices[lb.project_id],
@@ -310,6 +322,7 @@ class A10ControllerWorker(base_taskflow.BaseTaskFlowEngine):
                                                                         vthunder.compute_id)
         (flow, store) = self._lb_flows.get_delete_load_balancer_flow(lb, deleteCompute)
         store.update({constants.LOADBALANCER: lb,
+                      constants.VIP: lb.vip,
                       constants.SERVER_GROUP_ID: lb.server_group_id})
 
         delete_lb_tf = self._taskflow_load(flow, store=store)
@@ -340,6 +353,7 @@ class A10ControllerWorker(base_taskflow.BaseTaskFlowEngine):
         update_lb_tf = self._taskflow_load(
             self._lb_flows.get_update_load_balancer_flow(),
             store={constants.LOADBALANCER: lb,
+                   constants.VIP: lb.vip,
                    constants.LISTENERS: listeners,
                    constants.UPDATE_DICT: load_balancer_updates})
 
@@ -372,7 +386,13 @@ class A10ControllerWorker(base_taskflow.BaseTaskFlowEngine):
         load_balancer = pool.load_balancer
 
         topology = CONF.a10_controller_worker.loadbalancer_topology
-        if member.project_id in CONF.hardware_thunder.devices:
+        parent_project_list = utils.get_parent_project_list()
+        member_parent_proj = utils.get_parent_project(
+            member.project_id)
+
+        if (member.project_id in parent_project_list or
+                (member_parent_proj and member_parent_proj in parent_project_list)
+                or member.project_id in CONF.hardware_thunder.devices):
             create_member_tf = self._taskflow_load(
                 self._member_flows.get_rack_vthunder_create_member_flow(),
                 store={
@@ -382,7 +402,8 @@ class A10ControllerWorker(base_taskflow.BaseTaskFlowEngine):
                     constants.POOL: pool})
         else:
             create_member_tf = self._taskflow_load(self._member_flows.
-                                                   get_create_member_flow(topology=topology),
+                                                   get_create_member_flow(
+                                                       topology=topology),
                                                    store={constants.MEMBER: member,
                                                           constants.LISTENERS:
                                                           listeners,
