@@ -29,19 +29,28 @@ LOG = logging.getLogger(__name__)
 
 class LoadBalancerParent(object):
 
-    def set(self, set_method, loadbalancer):
+    def set(self, set_method, loadbalancer, **kwargs):
         status = self.axapi_client.slb.UP
         if not loadbalancer.provisioning_status:
             status = self.axapi_client.slb.DOWN
         vip_meta = utils.meta(loadbalancer, 'virtual_server', {})
         arp_disable = CONF.slb.arp_disable
         vrid = CONF.slb.default_virtual_server_vrid
+        desc = loadbalancer.description
+        if not desc:
+            desc = None
+        elif str(desc).isspace() or not str(desc):
+            desc = ""
+        else:
+            desc = '"{}"'.format(desc)
 
         set_method(
             loadbalancer.id,
             loadbalancer.vip.ip_address,
             arp_disable=arp_disable,
+            description=desc,
             status=status, vrid=vrid,
+            port_list=kwargs.get('port_list'),
             axapi_body=vip_meta)
 
 
@@ -75,12 +84,13 @@ class DeleteVirtualServerTask(task.Task):
 
     @axapi_client_decorator
     def execute(self, loadbalancer, vthunder):
-        try:
-            self.axapi_client.slb.virtual_server.delete(loadbalancer.id)
-            LOG.debug("Successfully deleted load balancer: %s", loadbalancer.id)
-        except (acos_errors.ACOSException, exceptions.ConnectionError) as e:
-            LOG.exception("Failed to delete load balancer: %s", loadbalancer.id)
-            raise e
+        if vthunder:
+            try:
+                self.axapi_client.slb.virtual_server.delete(loadbalancer.id)
+                LOG.debug("Successfully deleted load balancer: %s", loadbalancer.id)
+            except (acos_errors.ACOSException, exceptions.ConnectionError) as e:
+                LOG.exception("Failed to delete load balancer: %s", loadbalancer.id)
+                raise e
 
 
 class UpdateVirtualServerTask(LoadBalancerParent, task.Task):
@@ -89,7 +99,10 @@ class UpdateVirtualServerTask(LoadBalancerParent, task.Task):
     @axapi_client_decorator
     def execute(self, loadbalancer, vthunder):
         try:
-            self.set(self.axapi_client.slb.virtual_server.update, loadbalancer)
+            port_list = self.axapi_client.slb.virtual_server.get(
+                loadbalancer.id)['virtual-server'].get('port-list')
+            self.set(self.axapi_client.slb.virtual_server.replace, loadbalancer,
+                     port_list=port_list)
             LOG.debug("Successfully updated load balancer: %s", loadbalancer.id)
         except (acos_errors.ACOSException, exceptions.ConnectionError) as e:
             LOG.exception("Failed to update load balancer: %s", loadbalancer.id)
